unit TransBmp2;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Menus;

type
  TTransBmp2 = class(TGraphicControl)
  private
    FBitmap, FMaskBitmap: TBitmap;
    FRect: TRect;
    FTransColor: TColor;
  protected
    procedure SetBitmap(Bmp: TBitmap);
    procedure SetupMask(Source, Mask: TBitmap);
    procedure SetTransColor(Val: TColor);
    procedure PaintRect(const ARect: TRect;
      SourceCanvas, MaskCanvas, DestCanvas: TCanvas);
    property MaskBitmap: TBitmap read FMaskBitmap write FMaskBitmap;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
    procedure Loaded; override;
  published
    property Bitmap: TBitmap read FBitmap write SetBitmap;
    property TransColor: TColor read FTransColor write SetTransColor default clBlack;
    property DragCursor;
    property DragMode;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
  end;

procedure Register;

implementation

constructor TTransBmp2.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FBitmap := TBitmap.Create;
  FMaskBitmap := TBitmap.Create;
  FMaskBitmap.MonoChrome := True;
  FMaskBitmap.Canvas.Brush.Color := clWhite;
  { Set arbitrary default, otherwise it will be 0x0 }
  Height := 40;
  Width := 40;
end;

destructor TTransBmp2.Destroy;
begin
  FBitmap.Free;
  FMaskBitmap.Free;
  inherited Destroy;
end;

procedure TTransBmp2.Loaded;
begin
  inherited Loaded;
  SetupMask(FBitmap, FMaskBitmap);
end;

procedure TTransBmp2.SetupMask(Source, Mask: TBitmap);
begin
  Mask.Width  := Source.Width;
  Mask.Height := Source.Height;
  FRect := Rect(0, 0, Source.Width, Source.Height);
  { Create mask }
  Mask.Canvas.CopyMode := cmSrcCopy;
  SetBkColor(Source.Canvas.Handle, FTransColor);
  if (FBitmap.Height > 0) and (FBitmap.Width > 0) then
    Mask.Canvas.CopyRect(FRect, Source.Canvas, FRect);
end;

procedure TTransBmp2.SetTransColor(Val: TColor);
begin
  if Val <> FTransColor then
  begin
    FTransColor := Val;
    { Ensure mask is regenerated with new idea of transparent colour }
    SetupMask(FBitmap, FMaskBitmap);
    Invalidate;
  end;
end;

procedure TTransBmp2.SetBitmap(Bmp: TBitmap);
begin
  FBitmap.Assign(Bmp);
  if Assigned(Bmp) then
  begin
    Height := Bmp.Height;
    Width  := Bmp.Width;
    { Get default transparent colour (bottom left pixel). The And op }
    { gets rid of leading non-RGB byte which indiciates palette index }
    FTransColor := FBitmap.TransparentColor and clWhite;
    SetupMask(FBitmap, FMaskBitmap);
  end;
  Invalidate;
end;

procedure TTransBmp2.Paint;
var
  crBack, crText: TColorRef;
begin
  with Canvas do
  begin
    crBack := SetBkColor(Handle, clWhite);
    crText := SetTextColor(Handle, clBlack);
    CopyMode := cmSrcInvert;
    CopyRect(FRect, FBitmap.Canvas, FRect);
    CopyMode := cmSrcAnd;
    CopyRect(FRect, FMaskBitmap.Canvas, FRect);
    CopyMode := cmSrcInvert;
    CopyRect(FRect, FBitmap.Canvas, FRect);
    SetBkColor(Handle, crBack);
    SetTextColor(Handle, crText);
  end;

  { In case of confusion, here is the un-with'd version }

{ crBack := SetBkColor(Canvas.Handle, clWhite);
  crText := SetTextColor(Canvas.Handle, clBlack);
  Canvas.CopyMode := cmSrcInvert;
  Canvas.CopyRect(FRect, FBitmap.Canvas, FRect);
  Canvas.CopyMode := cmSrcAnd;
  Canvas.CopyRect(FRect, FMaskBitmap.Canvas, FRect);
  Canvas.CopyMode := cmSrcInvert;
  Canvas.CopyRect(FRect, FBitmap.Canvas, FRect);
  SetBkColor(Canvas.Handle, crBack);
  SetTextColor(Canvas.Handle, crText);}
end;

procedure Register;
begin
  RegisterComponents('Samples', [TTransBmp2]);
end;

end.
